/*
 * Copyright (C) 2006 JasperSoft http://www.jaspersoft.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; and without the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.txt 
 * or write to:
 * 
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330,
 * Boston, MA  USA  02111-1307
 */

package com.jaspersoft.jasperserver.ws.axis2.repository;

import java.io.IOException;
import java.util.Map;

import javax.activation.DataHandler;
import javax.xml.soap.SOAPException;

import org.apache.axis.attachments.AttachmentPart;

import com.jaspersoft.jasperserver.api.metadata.common.domain.FileResource;
import com.jaspersoft.jasperserver.api.metadata.common.domain.FileResourceData;
import com.jaspersoft.jasperserver.api.metadata.common.domain.Resource;
import com.jaspersoft.jasperserver.api.metadata.xml.domain.impl.Argument;
import com.jaspersoft.jasperserver.api.metadata.xml.domain.impl.ResourceDescriptor;
import com.jaspersoft.jasperserver.ws.axis2.RepositoryServiceContext;
import com.jaspersoft.jasperserver.ws.axis2.ResourceDataSource;
import com.jaspersoft.jasperserver.ws.axis2.ResultAttachments;
import com.jaspersoft.jasperserver.ws.axis2.WSException;

/**
 * @author gtoffoli
 * @version $Id: FileResourceHandler.java 9948 2007-09-07 12:52:21Z lucian $
 */
public class FileResourceHandler extends RepositoryResourceHandler {

	public Class getResourceType() {
		return FileResource.class;
	}

	protected void doDescribe(Resource resource, ResourceDescriptor descriptor,
			Map arguments, RepositoryServiceContext serviceContext) {
		FileResource fileResource = (FileResource) resource;
		descriptor.setWsType(fileResource.getFileType());
		descriptor.setHasData(fileResource.hasData());
		descriptor.setIsReference(fileResource.isReference());
		if (descriptor.getIsReference()) {
			descriptor.setReferenceUri(fileResource.getReferenceURI());
		}

		String resourceType = fileResource.getFileType();

		if (descriptor.getIsReference()) {
			descriptor.setWsType(ResourceDescriptor.TYPE_REFERENCE);
		} else if (resourceType == null)
			descriptor.setWsType(ResourceDescriptor.TYPE_UNKNOW);
		else if (resourceType.equals("jrxml"))
			descriptor.setWsType(ResourceDescriptor.TYPE_JRXML);
		else if (resourceType.equals("img"))
			descriptor.setWsType(ResourceDescriptor.TYPE_IMAGE);
		else if (resourceType.equals("font"))
			descriptor.setWsType(ResourceDescriptor.TYPE_FONT);
		else if (resourceType.equals("jar"))
			descriptor.setWsType(ResourceDescriptor.TYPE_CLASS_JAR);
		else if (resourceType.equals("prop"))
			descriptor.setWsType(ResourceDescriptor.TYPE_RESOURCE_BUNDLE);
	}

	protected void updateResource(Resource resource,
			ResourceDescriptor descriptor, RepositoryServiceContext serviceContext) throws WSException {
		FileResource fileResource = (FileResource) resource;

		String wsType = descriptor.getWsType();

		if (wsType.equals(ResourceDescriptor.TYPE_REFERENCE)
				|| descriptor.getIsReference()) {
			// check if the reference uri is valid...
			String referenceUri = descriptor.getReferenceUri();

			try {
				serviceContext.getRepository().getResource(null,
						referenceUri);
			} catch (Exception ex) {
				throw new WSException(WSException.GENERAL_ERROR2,
						serviceContext.getMessage(
								"webservices.error.resourceNotFoundOrInvalid",
								new Object[] { referenceUri }));
			}

			fileResource.setReferenceURI(referenceUri);
		} else {
			if (wsType.equals(ResourceDescriptor.TYPE_IMAGE))
				fileResource.setFileType(FileResource.TYPE_IMAGE);
			else if (wsType.equals(ResourceDescriptor.TYPE_FONT))
				fileResource.setFileType(FileResource.TYPE_FONT);
			else if (wsType.equals(ResourceDescriptor.TYPE_CLASS_JAR))
				fileResource.setFileType(FileResource.TYPE_JAR);
			else if (wsType.equals(ResourceDescriptor.TYPE_JRXML))
				fileResource.setFileType(FileResource.TYPE_JRXML);
			else if (wsType.equals(ResourceDescriptor.TYPE_RESOURCE_BUNDLE))
				fileResource.setFileType(FileResource.TYPE_RESOURCE_BUNDLE);

			if (descriptor.getHasData()) {
				try {
					// Save the temporary file....
					AttachmentPart[] parts = serviceContext
							.getMessageAttachments();
					if (parts.length >= 1) {
						DataHandler actualDH = parts[0].getDataHandler();

						// Save the content in the file...
						fileResource.readData(actualDH.getInputStream());
					}
				} catch (SOAPException e) {
					throw new WSException(e);
				} catch (IOException e) {
					throw new WSException(e);
				}
			}
		}
	}

	public void getAttachments(Resource resource, Map arguments,
			ResourceDescriptor descriptor,
			ResultAttachments attachments, RepositoryServiceContext serviceContext) {
		super.getAttachments(resource, arguments, descriptor, attachments, serviceContext);
		
        if (arguments.containsKey(Argument.NO_RESOURCE_DATA_ATTACHMENT))
        {
            descriptor.setHasData(false);
        }
        else
        {
            //Retrieve resource data...
        	descriptor.setHasData(true);
        	FileResourceData fileData = serviceContext.getRepository().getResourceData(null, descriptor.getUriString());
        	ResourceDataSource attachmentSource = new ResourceDataSource("", fileData);
        	attachments.addAttachment("attachment", attachmentSource);
        }
	}
	
}
